# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK, \
	Entry, Label, Button, STOCK_ADD, ListStore, TreeView, CellRendererText, ScrolledWindow, POLICY_AUTOMATIC, SHADOW_IN, \
	STOCK_GO_UP, STOCK_GO_DOWN, STOCK_DELETE, VButtonBox, BUTTONBOX_START, Table, EXPAND, FILL, combo_box_entry_new_text, \
	CellRendererToggle, CellRendererCombo, accelerator_parse, POLICY_NEVER, HSeparator

from builder.Profile import Profile
from builder.Job import Job

class ProfileDialog(Dialog):
	"""
	The dialog for configuring or adding a build profile
	"""
	def __init__(self, parent):
		Dialog.__init__(self, "", parent, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, 
					(STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		#
		# help panel
		#
		self.__lbl_hint = Label()
		self.__lbl_hint.set_line_wrap(True)
		self.__lbl_hint.set_alignment(0, .5)
		
		sep = HSeparator()
		
		#
		# name
		#
		self.entry_name = Entry()
		
		lbl_name = Label()
		lbl_name.set_markup("<b>_Name</b>")
		lbl_name.set_alignment(0, .5)
		lbl_name.set_use_underline(True)
		lbl_name.set_mnemonic_widget(self.entry_name)
		
		#
		# jobs
		#
		lbl_info = Label("<small>Valid tokens are <b>%f</b>: filename, <b>%e</b>: filename without .tex suffix, <b>%p</b>: path of the LaTeX plugin</small>")
		lbl_info.set_use_markup(True)
		lbl_info.set_line_wrap(True)
		lbl_info.set_alignment(0, .5)
		
		self.cmb_command = combo_box_entry_new_text()
		for item in ["latex -interaction batchmode -file-line-error %f", "dvipdf %e.dvi"]:
			self.cmb_command.append_text(item)
		
		lbl_info_jobs = Label("<small>The build process only stops at failed jobs if they are a <i>stopper</i>. The <i>processor</i> is used to generate messages from the job output.</small>")
		lbl_info_jobs.set_use_markup(True)
		lbl_info_jobs.set_line_wrap(True)
		lbl_info_jobs.set_alignment(0, .5)
		
		lbl_commands = Label()
		lbl_commands.set_markup("<b>_Jobs</b>")
		lbl_commands.set_alignment(0, .5)
		lbl_commands.set_use_underline(True)
		lbl_commands.set_mnemonic_widget(self.cmb_command)
		
		btn_add = Button(stock=STOCK_ADD)
		btn_add.connect("clicked", lambda x: self.__add_job())
		
		scr_jobs = ScrolledWindow()
		scr_jobs.set_policy(POLICY_NEVER, POLICY_AUTOMATIC);
		scr_jobs.set_shadow_type(SHADOW_IN)
		scr_jobs.add(self.__get_view_jobs())
		
		btn_down = Button(stock=STOCK_GO_DOWN)
		btn_down.connect("clicked", lambda x: self.__move_down_command())
		
		btn_delete = Button(stock=STOCK_DELETE)
		btn_delete.connect("clicked", lambda x: self.__delete_job())
		
		buttons = VButtonBox()
		buttons.set_layout(BUTTONBOX_START)
		buttons.pack_start(btn_down)
		buttons.pack_start(btn_delete)
		
		## view command
		self.entry_view_command = Entry()
		
		lbl_view = Label()
		lbl_view.set_markup("<b>_Viewing Command</b>")
		lbl_view.set_alignment(0, .5)
		lbl_view.set_use_underline(True)
		lbl_view.set_mnemonic_widget(self.entry_view_command)
		
		## accelerator
		self.entry_accel = Entry()
		self.entry_accel.connect("changed", lambda x: self.__validate())
		
		lbl_accel = Label()
		lbl_accel.set_markup("<b>Accelerator</b>")
		lbl_accel.set_alignment(0, .5)
		lbl_accel.set_use_underline(True)
		lbl_accel.set_mnemonic_widget(self.entry_accel)
		
		lbl_accel_info = Label()
		lbl_accel_info.set_markup("<small>An accelerator like <b>&lt;Shift&gt;&lt;Alt&gt;F1</b> or <b>&lt;Ctrl&gt;a</b></small>")
		lbl_accel_info.set_alignment(0, .5)
		
		## pack
		table = Table()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(self.__lbl_hint, 0, 3, 0, 1)
		table.attach(sep, 0, 3, 1, 2)
		
		table.attach(lbl_name, 0, 3, 2, 3, EXPAND|FILL, FILL)
		table.attach(self.entry_name, 1, 3, 3, 4, EXPAND|FILL, FILL)
		table.attach(lbl_commands, 0, 3, 4, 5, EXPAND|FILL, FILL)
		table.attach(lbl_info, 1, 3, 5, 6, EXPAND|FILL, FILL)
		table.attach(self.cmb_command, 1, 2, 6, 7, EXPAND|FILL, 0)
		table.attach(btn_add, 2, 3, 6, 7, FILL, FILL)
		table.attach(lbl_info_jobs, 1, 3, 7, 8, EXPAND|FILL, FILL)
		table.attach(scr_jobs, 1, 2, 8, 9)
		table.attach(buttons, 2, 3, 8, 9, FILL, FILL)
		table.attach(lbl_view, 0, 3, 9, 10, EXPAND|FILL, FILL)
		table.attach(self.entry_view_command, 1, 3, 10, 11, EXPAND|FILL, FILL)
		table.attach(lbl_accel, 0, 3, 11, 12, EXPAND|FILL, FILL)
		table.attach(lbl_accel_info, 1, 3, 12, 13, EXPAND|FILL, FILL)
		table.attach(self.entry_accel, 1, 3, 13, 14, EXPAND|FILL, FILL)
		
		table.set_col_spacing(0, 15)
		table.set_row_spacing(3, 15)
		table.set_row_spacing(8, 15)
		table.set_row_spacing(10, 15)
		
		self.vbox.add(table)
		
		# find okay button
		#
		# TODO: do this better
		#
		self.__okay_button = None
		for button in self.action_area.get_children():
			if self.get_response_for_widget(button) == RESPONSE_OK:
				self.__okay_button = button
	
	def __validate(self):
		"""
		Validate the form
		"""
		errors = []
		
		#
		# accelerator
		#
		accel_is_valid = True
		accel = self.entry_accel.get_text()
		if len(accel):
			keyval, modifier = accelerator_parse(accel)
			if not keyval or not modifier:
				accel_is_valid = False
			
		if not accel_is_valid:
			errors.append("The entered accelerator is invalid.")
		#
		# jobs
		#
		if len(self.__store_jobs) == 0:
			errors.append("You have not specified any jobs.")
		
		if len(errors):
			self.__okay_button.set_sensitive(False)
		else:
			self.__okay_button.set_sensitive(True)
		
		if len(errors) == 1:
			self.__lbl_hint.set_markup(errors[0])
		elif len(errors) > 1:
			self.__lbl_hint.set_markup("\n".join(["• %s" % e for e in errors]))
		else:
			self.__lbl_hint.set_markup("Remember to run all commands in batch mode (e.g. append <tt>-interaction batchmode</tt> to <tt>latex</tt>)")
	
	def __get_view_jobs(self):
		self.__store_jobs = ListStore(str, bool, str) 	# command, stoppper, processor
		
		self.__view_jobs = TreeView(self.__store_jobs)
		
		toggle_renderer = CellRendererToggle()
		toggle_renderer.connect("toggled", self.__stopper_toggled)
		
		text_renderer = CellRendererText()
		text_renderer.connect("edited", self.__command_edited)
		
		self.__view_jobs.insert_column_with_attributes(-1, "Command", text_renderer, text=0, editable=True)
		
		self.__view_jobs.insert_column_with_attributes(-1, "Stopper", toggle_renderer, active=1)
		
		self.__processor_model = ListStore(str)
		self.__processor_model.append(["default"])
		self.__processor_model.append(["rubber"])
		self.__processor_model.append(["latex"])
		
		combo_renderer = CellRendererCombo()
		combo_renderer.set_property("editable", True)
		combo_renderer.set_property("model", self.__processor_model)
		combo_renderer.set_property("text_column", 0)
		combo_renderer.set_property("has_entry", False)
		
		combo_renderer.connect("edited", self.__processor_edited)
		
		self.__view_jobs.insert_column_with_attributes(-1, "Processor", combo_renderer, text=2)
		
		return self.__view_jobs
	
	def __command_edited(self, renderer, path, text):
		self.__store_jobs.set(self.__store_jobs.get_iter_from_string(path), 0, text)
	
	def __processor_edited(self, renderer, path, text):
		self.__store_jobs.set(self.__store_jobs.get_iter_from_string(path), 2, text)
	
	def __stopper_toggled(self, renderer, path):
		value = self.__store_jobs.get(self.__store_jobs.get_iter_from_string(path), 1)[0]
		self.__store_jobs.set(self.__store_jobs.get_iter_from_string(path), 1, not value)
	
	def __move_down_command(self):
		store, it = self.__view_jobs.get_selection().get_selected()
		next_it = store.iter_next(it)
		if (next_it):
			store.swap(it, next_it)
	
	def __add_job(self):
		command = self.cmb_command.get_active_text()
		if len(command):
			self.__store_jobs.append([command, True, "default"])
			self.__validate()

	def __delete_job(self):
		store, it = self.__view_jobs.get_selection().get_selected()
		if it:
			store.remove(it)
			self.__validate()
	
	def run(self):
		self.show_all()
		return Dialog.run(self)
	
	def get_profile(self):
		profile = Profile()
		profile.name = self.entry_name.get_text()
		profile.viewer = self.entry_view_command.get_text()
		profile.accel = self.entry_accel.get_text()
		
		for row in self.__store_jobs:
			job = Job()
			job.command = row[0]
			job.stopper = row[1]
			job.processor = row[2]
			
			profile.jobs.append(job)
		
		return profile
	
	def set_profile(self, profile):
		if profile:
			self.entry_name.set_text(profile.name)
			
			self.__store_jobs.clear()
			for job in profile.jobs:
				self.__store_jobs.append([job.command, job.stopper, job.processor])
				
			self.entry_view_command.set_text(profile.viewer)
			self.entry_accel.set_text(profile.accel)
			
			self.set_title("Configure Profile %s" % profile.name)
			self.entry_name.set_sensitive(False)
		else:
			self.set_title("Create Profile")
			self.entry_name.set_sensitive(True)
			self.__okay_button.set_sensitive(False)
		
		self.__validate()
		
