# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from os import WIFEXITED, WEXITSTATUS, WIFSTOPPED, WSTOPSIG, WIFSIGNALED, WTERMSIG
from LaTeXPlugin.util import show_warning

class Processor(object):
	def __init__(self):
		self.filename = None
		self.condition = None
	
	def get_messages(self):
		"""
		Return a list of messages
		"""
		return []
	
	def get_summary(self):
		"""
		Return a summarizing message
		"""
		if self.condition:
			if WIFEXITED(self.condition):
				code = WEXITSTATUS(self.condition)
				if code == 127:
					show_warning("A command could not be found. Please ensure that you have a LaTeX distribution and the rubber tool installed.")
				return "failed returning %s" % code
			elif WIFSTOPPED(self.condition):
				return "stopped by signal %s" % WSTOPSIG(self.condition)
			elif WIFSIGNALED(self.condition):
				return "terminated by signal %s" % WTERMSIG(self.condition)
			return "illegal condition"
		else:
			return "successful"