# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from re import compile, MULTILINE
from os import popen

from Message import *
from Processor import Processor
from LaTeXPlugin.Logger import debug

class RubberProcessor(Processor):
	"""
	This generates messages from the output of rubber-info
	"""
	#
	# TODO: falls rubber nicht installiert ist, entspr. Message generieren
	# vgl. Builder.__rubber_installed()
	#
	# TODO: die Messages insgesamt verbessern: weniger pro Zeile
	#
	def __init__(self):
		Processor.__init__(self)
		debug(self, "__init__")
		self.__pattern = compile(r"(?P<file>[a-zA-Z0-9./_-]+)(:(?P<line>[0-9\-]+))?:(?P<text>.*?)$", 
								MULTILINE)
	
	def get_messages(self):
		"""
		Retrieve messages for a document that has been built with rubber
		"""
		
#		if condition:
#			# rubber failed, return errors warnings
#			output = popen("rubber-info --errors \"%s\" 2>&1" % input_filename).read()
#			severity = SEVERITY_ERROR
#		else:
#			# rubber succeeded, return only warnings
#			output = popen("rubber-info --warnings \"%s\" 2>&1" % input_filename).read()
#			severity = SEVERITY_WARNING
		
		# TODO: improve this
		if self.condition:
			severity = ERROR
			#output = popen("rubber-info \"%s\" 2>&1" % self.filename).read()
		else:
			severity = WARNING
			#output = popen("rubber-info \"%s\" 2>&1" % self.filename).read()
		
		# generate messages
		#
		# TODO: add --boxes if ShowBoxWarnings is in settings
		#
		output = popen("rubber-info \"%s\" 2>&1" % self.filename).read()
		
		messages = []
		
		for match in self.__pattern.finditer(output):
			line_from, line_to = None, None
			
			line = match.group("line")
			if line:
				parts = line.split("-")
				line_from = int(parts[0]) - 1
				if len(parts) > 1:
					line_to = int(parts[1]) - 1
			
			messages.append(Message(match.group("file"), line_from, line_to, match.group("text"), severity))
		
		return messages
	