# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import RESPONSE_OK

from LaTeXPlugin.document.DocumentHelper import LANGUAGE_LATEX
from ListingDialog import ListingDialog
from ListingSourceBuilder import ListingSourceBuilder
from LanguagesParser import LanguagesParser
from LaTeXPlugin.Installation import Installation

class ListingController(object):
	def __init__(self, window_helper):
		self.__window_helper = window_helper
		self.__languages = None
		self.__dialog = None
		self.__source_builder = None
	
	def run_dialog(self):
		helper = self.__window_helper.document_helper_manager.active_helper
		if helper.get_language() != LANGUAGE_LATEX:
			return
		
		if not self.__languages:
			self.__languages = []
			parser = LanguagesParser()
			parser.parse(self.__languages, "%s/listings/languages.xml" % Installation().path)
		
		if not self.__dialog:
			self.__dialog = ListingDialog(self.__window_helper.window, self.__languages)
		
		if not self.__source_builder:
			self.__builder = ListingSourceBuilder()
		
		if self.__dialog.run() == RESPONSE_OK:
			self.__builder.language = self.__dialog.language
			self.__builder.dialect = self.__dialog.dialect
			self.__builder.filename = self.__dialog.filename
			self.__builder.tabsize = self.__dialog.tabsize
			
			helper.language_helper.insert_source(self.__builder.source)
		
		self.__dialog.hide()