# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from xml.dom.minidom import parse
from xml.parsers.expat import ExpatError

from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.Logger import error, debug
from LaTeXPlugin.util import str_bool

class Group(object):
	def __init__(self, label, visible):
		self.label = label
		self.visible = visible
		self.symbols = []
		
class Symbol(object):
	def __init__(self, source, icon):
		self.source = source
		self.icon = icon

class SymbolDefinition(object):
	"""
	A collection of symbols read from an XML file
	"""
	def __init__(self, filename):
		self.__filename = filename
		self.groups = []
		
		try:
			debug(self, "__init__", "creating DOM")
			self.__dom = parse(self.__filename)
			
			debug(self, "__init__", "processing DOM")
			for group_node in self.__dom.getElementsByTagName("g"):
				label = group_node.getAttribute("label")
				visible = str_bool(group_node.getAttribute("visible"))
				
				group = Group(label, visible)
				
				for symbol_node in group_node.getElementsByTagName("s"):
					source = symbol_node.getAttribute("source")
					icon = symbol_node.getAttribute("icon")
					
					symbol = Symbol(source, "%s/pixmaps/%s" % (Installation().path, icon))
					
					group.symbols.append(symbol)
				
				self.groups.append(group)
			
			debug(self, "__init__", "processing DOM done")
		except IOError:
			error(self, "__init__", "File not found: %s" % self.__filename)
		except ExpatError, s:
			error(self, "__init__", "Parse error: %s" % s)
	
	def __flush(self):
		try:
			out = open(self.__filename, "w+")
			out.write(self.__dom.toxml())
			out.close()
		except IOError, e:
			error(self, "__flush", e)
		
	def set_group_visible(self, group_label, visible):
		debug(self, "set_group_visible", "%s, %s" % (group_label, visible))
		
		# update model
		for group in self.groups:
			if group.label == group_label:
				group.visible = visible
				break
				
		# update DOM
		for group_node in self.__dom.getElementsByTagName("g"):
			if group_node.getAttribute("label") == group_label:
				group_node.setAttribute("visible", str(visible))
				self.__flush()
				return
		
		error(self, "set_group_visible", "Group %s not found" % group_label)
			
			